@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
+from cProfile import Profile
 from functools import wraps
 from os import environ
 from os.path import abspath, dirname
 from pipes import quote
 from sys import argv, exit, stderr, stdout
-from traceback import print_exc
+from traceback import format_exc, print_exc
 
 
 from ..exceptions import NoSuchRepository, MissingRepoDependency
@@ -101,21 +102,15 @@ def main(*args, **kwargs):
     if not hasattr(pargs, 'func'):
         parser_bw.print_help()
         exit(2)
+    if pargs.profile:
+        profile = Profile()
+        profile.enable()
 
     path = abspath(pargs.repo_path)
     io.debug_mode = pargs.debug
     io.activate()
     io.debug(_("invocation: {}").format(" ".join([force_text(arg) for arg in argv])))
 
-    if 'BWADDHOSTKEYS' in environ:  # TODO remove in 3.0.0
-        environ.setdefault('BW_ADD_HOST_KEYS', environ['BWADDHOSTKEYS'])
-    if 'BWCOLORS' in environ:  # TODO remove in 3.0.0
-        environ.setdefault('BW_COLORS', environ['BWCOLORS'])
-    if 'BWITEMWORKERS' in environ:  # TODO remove in 3.0.0
-        environ.setdefault('BW_ITEM_WORKERS', environ['BWITEMWORKERS'])
-    if 'BWNODEWORKERS' in environ:  # TODO remove in 3.0.0
-        environ.setdefault('BW_NODE_WORKERS', environ['BWNODEWORKERS'])
-
     environ.setdefault('BW_ADD_HOST_KEYS', "1" if pargs.add_ssh_host_keys else "0")
 
     if len(text_args) >= 1 and (
@@ -138,11 +133,17 @@ def main(*args, **kwargs):
                         "{x} {path} "
                         "is not a BundleWrap repository."
                     ).format(path=quote(abspath(pargs.repo_path)), x=red("!!!")))
+                    io.deactivate()
                     exit(1)
                 else:
                     path = dirname(path)
             except MissingRepoDependency as exc:
                 io.stderr(str(exc))
+                io.deactivate()
+                exit(1)
+            except Exception:
+                io.stderr(format_exc())
+                io.deactivate()
                 exit(1)
 
     # convert all string args into text
@@ -152,3 +153,6 @@ def main(*args, **kwargs):
         pargs.func(repo, text_pargs)
     finally:
         io.deactivate()
+        if pargs.profile:
+            profile.disable()
+            profile.dump_stats(pargs.profile)
