@@ -12,11 +12,12 @@ from textwrap import TextWrapper
 
 from bundlewrap.exceptions import BundleError, ItemDependencyError, FaultUnavailable
 from bundlewrap.utils import cached_property
-from bundlewrap.utils.statedict import diff_keys, diff_value, hash_statedict, validate_statedict
+from bundlewrap.utils.dicts import diff_keys, diff_value, hash_statedict, validate_statedict
 from bundlewrap.utils.text import force_text, mark_for_translation as _
 from bundlewrap.utils.text import blue, bold, italic, wrap_question
 from bundlewrap.utils.ui import io
 
+
 BUILTIN_ITEM_ATTRIBUTES = {
     'cascade_skip': None,
     'comment': None,
@@ -55,7 +56,7 @@ class ItemStatus(object):
     fixing and what's broken.
     """
 
-    def __init__(self, cdict, sdict):
+    def __init__(self, cdict, sdict, display_dicts):
         self.cdict = cdict
         self.sdict = sdict
         self.keys_to_fix = []
@@ -64,6 +65,12 @@ class ItemStatus(object):
         if not self.must_be_deleted and not self.must_be_created:
             self.keys_to_fix = diff_keys(cdict, sdict)
 
+        self.display_cdict, self.display_sdict, self.display_keys_to_fix = display_dicts(
+            copy(cdict),
+            copy(sdict),
+            copy(self.keys_to_fix),
+        )
+
     def __repr__(self):
         return "<ItemStatus correct:{}>".format(self.correct)
 
@@ -81,6 +88,24 @@ class Item(object):
     ITEM_ATTRIBUTES = {}
     ITEM_TYPE_NAME = None
     REQUIRED_ATTRIBUTES = []
+    SKIP_REASON_CMDLINE = 1
+    SKIP_REASON_DEP_FAILED = 2
+    SKIP_REASON_FAULT_UNAVAILABLE = 3
+    SKIP_REASON_INTERACTIVE = 4
+    SKIP_REASON_INTERACTIVE_ONLY = 5
+    SKIP_REASON_NO_TRIGGER = 6
+    SKIP_REASON_SOFTLOCK = 7
+    SKIP_REASON_UNLESS = 8
+    SKIP_REASON_DESC = {
+        SKIP_REASON_CMDLINE: _("cmdline"),
+        SKIP_REASON_DEP_FAILED: _("dependency failed"),
+        SKIP_REASON_FAULT_UNAVAILABLE: _("Fault unavailable"),
+        SKIP_REASON_INTERACTIVE: _("declined interactively"),
+        SKIP_REASON_INTERACTIVE_ONLY: _("interactive only"),
+        SKIP_REASON_NO_TRIGGER: _("not triggered"),
+        SKIP_REASON_SOFTLOCK: _("soft locked"),
+        SKIP_REASON_UNLESS: _("unless"),
+    }
     STATUS_OK = 1
     STATUS_FIXED = 2
     STATUS_FAILED = 3
@@ -88,6 +113,14 @@ class Item(object):
     STATUS_ACTION_SUCCEEDED = 5
     WHEN_CREATING_ATTRIBUTES = {}
 
+    @classmethod
+    def block_concurrent(cls, node_os, node_os_version):
+        """
+        Return a list of item types that cannot be applied in parallel
+        with this item type.
+        """
+        return []
+
     def __init__(
         self,
         bundle,
@@ -242,29 +275,31 @@ class Item(object):
 
     @cached_property
     def cached_unless_result(self):
-        if self.unless and not self.cached_status.correct:
+        """
+        Returns True if 'unless' wants to skip this item.
+        """
+        if self.unless and (self.ITEM_TYPE_NAME == 'action' or not self.cached_status.correct):
             unless_result = self.node.run(self.unless, may_fail=True)
             return unless_result.return_code == 0
         else:
             return False
 
-    def _precedes_incorrect_item(self, interactive=False):
+    def _triggers_preceding_items(self, interactive=False):
         """
-        Returns True if this item precedes another and the triggering
-        item is in need of fixing.
+        Preceding items will execute this to figure out if they're
+        triggered.
         """
-        for item in self._precedes_items:
-            if item._precedes_incorrect_item():
-                return True
         if self.cached_unless_result:
-            # triggering item failed unless, so there is nothing to do
+            # 'unless' says we don't need to run
             return False
         if self.ITEM_TYPE_NAME == 'action':
+            # so we have an action where 'unless' says it must be run
+            # but the 'interactive' attribute might still override that
             if self.attributes['interactive'] != interactive or \
                     self.attributes['interactive'] is None:
-                return False
-            else:
                 return True
+            else:
+                return False
         return not self.cached_status.correct
 
     def _prepare_deps(self, items):
@@ -309,12 +344,24 @@ class Item(object):
         return False
 
     def _test(self):
-        if self._faults_missing_for_attributes:
-            self._raise_for_faults()
-        return self.test()
+        with io.job(_("{node}  {bundle}  {item}").format(
+            bundle=bold(self.bundle.name),
+            item=self.id,
+            node=bold(self.node.name),
+        )):
+            if self._faults_missing_for_attributes:
+                self._raise_for_faults()
+            return self.test()
 
     @classmethod
     def _validate_attribute_names(cls, bundle, item_id, attributes):
+        if not isinstance(attributes, dict):
+            raise BundleError(_(
+                "invalid item '{item}' in bundle '{bundle}': not a dict"
+            ).format(
+                item=item_id,
+                bundle=bundle.name,
+            ))
         invalid_attributes = set(attributes.keys()).difference(
             set(cls.ITEM_ATTRIBUTES.keys()).union(
                 set(BUILTIN_ITEM_ATTRIBUTES.keys())
@@ -379,7 +426,6 @@ class Item(object):
             self.node,
             self,
         )
-        keys_to_fix = None
         status_code = None
         status_before = None
         status_after = None
@@ -390,25 +436,37 @@ class Item(object):
                 "autoskip matches {item} on {node}"
             ).format(item=self.id, node=self.node.name))
             status_code = self.STATUS_SKIPPED
-            keys_to_fix = [_("cmdline")]
+            skip_reason = self.SKIP_REASON_CMDLINE
 
         if self._skip_with_soft_locks(my_soft_locks, other_peoples_soft_locks):
             status_code = self.STATUS_SKIPPED
-            keys_to_fix = [_("soft locked")]
+            skip_reason = self.SKIP_REASON_SOFTLOCK
+
+        for item in self._precedes_items:
+            if item._triggers_preceding_items(interactive=interactive):
+                io.debug(_(
+                    "preceding item {item} on {node} has been triggered by {other_item}"
+                ).format(item=self.id, node=self.node.name, other_item=item.id))
+                self.has_been_triggered = True
+                break
+            else:
+                io.debug(_(
+                    "preceding item {item} on {node} has NOT been triggered by {other_item}"
+                ).format(item=self.id, node=self.node.name, other_item=item.id))
 
         if self.triggered and not self.has_been_triggered and status_code is None:
             io.debug(_(
                 "skipping {item} on {node} because it wasn't triggered"
             ).format(item=self.id, node=self.node.name))
             status_code = self.STATUS_SKIPPED
-            keys_to_fix = [_("not triggered")]
+            skip_reason = self.SKIP_REASON_NO_TRIGGER
 
         if status_code is None and self.cached_unless_result and status_code is None:
             io.debug(_(
                 "'unless' for {item} on {node} succeeded, not fixing"
             ).format(item=self.id, node=self.node.name))
             status_code = self.STATUS_SKIPPED
-            keys_to_fix = ["unless"]
+            skip_reason = self.SKIP_REASON_UNLESS
 
         if self._faults_missing_for_attributes and status_code is None:
             if self.error_on_missing_fault:
@@ -425,7 +483,7 @@ class Item(object):
                     node=self.node.name,
                 ))
                 status_code = self.STATUS_SKIPPED
-                keys_to_fix = [_("Fault unavailable")]
+                skip_reason = self.SKIP_REASON_FAULT_UNAVAILABLE
 
         if status_code is None:
             try:
@@ -443,22 +501,17 @@ class Item(object):
                         node=self.node.name,
                     ))
                     status_code = self.STATUS_SKIPPED
-                    keys_to_fix = [_("Fault unavailable")]
+                    skip_reason = self.SKIP_REASON_FAULT_UNAVAILABLE
             else:
                 if status_before.correct:
                     status_code = self.STATUS_OK
 
         if status_code is None:
-            keys_to_fix = self.display_keys(
-                copy(self.cached_cdict),
-                copy(status_before.sdict),
-                status_before.keys_to_fix[:],
-            )
             if not interactive:
-                with io.job(_("  {node}  {bundle}  {item}  fixing...").format(
-                    bundle=self.bundle.name,
+                with io.job(_("{node}  {bundle}  {item}").format(
+                    bundle=bold(self.bundle.name),
                     item=self.id,
-                    node=self.node.name,
+                    node=bold(self.node.name),
                 )):
                     self.fix(status_before)
             else:
@@ -467,12 +520,11 @@ class Item(object):
                 elif status_before.must_be_deleted:
                     question_text = _("Found on node. Will be removed.")
                 else:
-                    cdict, sdict = self.display_dicts(
-                        copy(self.cached_cdict),
-                        copy(status_before.sdict),
-                        keys_to_fix,
+                    question_text = self.ask(
+                        status_before.display_cdict,
+                        status_before.display_sdict,
+                        status_before.display_keys_to_fix,
                     )
-                    question_text = self.ask(cdict, sdict, keys_to_fix)
                 if self.comment:
                     question_text += format_comment(self.comment)
                 question = wrap_question(
@@ -493,35 +545,32 @@ class Item(object):
                     ),
                 )
                 if answer:
-                    with io.job(_("  {node}  {bundle}  {item}  fixing...").format(
-                        bundle=self.bundle.name,
+                    with io.job(_("{node}  {bundle}  {item}").format(
+                        bundle=bold(self.bundle.name),
                         item=self.id,
-                        node=self.node.name,
+                        node=bold(self.node.name),
                     )):
                         self.fix(status_before)
                 else:
                     status_code = self.STATUS_SKIPPED
-                    keys_to_fix = [_("interactive")]
+                    skip_reason = self.SKIP_REASON_INTERACTIVE
 
         if status_code is None:
             status_after = self.get_status(cached=False)
             status_code = self.STATUS_FIXED if status_after.correct else self.STATUS_FAILED
 
-        if status_code == self.STATUS_SKIPPED:
-            # can't use else for this because status_before is None
-            changes = keys_to_fix
+        if status_code == self.STATUS_OK:
+            details = None
+        elif status_code == self.STATUS_SKIPPED:
+            details = skip_reason
         elif status_before.must_be_created:
-            changes = True
+            details = True
         elif status_before.must_be_deleted:
-            changes = False
+            details = False
         elif status_code == self.STATUS_FAILED:
-            changes = self.display_keys(
-                self.cached_cdict.copy(),
-                status_after.sdict.copy(),
-                status_after.keys_to_fix[:],
-            )
+            details = status_after.display_keys_to_fix
         else:
-            changes = keys_to_fix
+            details = status_before.display_keys_to_fix
 
         self.node.repo.hooks.item_apply_end(
             self.node.repo,
@@ -532,7 +581,7 @@ class Item(object):
             status_before=status_before,
             status_after=status_after,
         )
-        return (status_code, changes)
+        return (status_code, details)
 
     def ask(self, status_should, status_actual, relevant_keys):
         """
@@ -607,14 +656,14 @@ class Item(object):
         Returns an ItemStatus instance describing the current status of
         the item on the actual node.
         """
-        with io.job(_("  {node}  {bundle}  {item}  checking...").format(
-            bundle=self.bundle.name,
+        with io.job(_("{node}  {bundle}  {item}").format(
+            bundle=bold(self.bundle.name),
             item=self.id,
-            node=self.node.name,
+            node=bold(self.node.name),
         )):
             if not cached:
                 del self._cache['cached_sdict']
-            return ItemStatus(self.cached_cdict, self.cached_sdict)
+            return ItemStatus(self.cached_cdict, self.cached_sdict, self.display_dicts)
 
     def hash(self):
         return hash_statedict(self.cached_cdict)
@@ -632,22 +681,12 @@ class Item(object):
     def display_dicts(self, cdict, sdict, keys):
         """
         Given cdict and sdict as implemented above, modify them to
-        better suit interactive presentation. The keys parameter is the
-        return value of display_keys (see below) and provided for
-        reference only.
+        better suit interactive presentation. The keys parameter is a
+        list of keys whose values differ between cdict and sdict.
 
         MAY be overridden by subclasses.
         """
-        return (cdict, sdict)
-
-    def display_keys(self, cdict, sdict, keys):
-        """
-        Given a list of keys whose values differ between cdict and
-        sdict, modify them to better suit presentation to the user.
-
-        MAY be overridden by subclasses.
-        """
-        return keys
+        return (cdict, sdict, keys)
 
     def patch_attributes(self, attributes):
         """
