@@ -49,29 +49,41 @@ class Action(Item):
                     item=self.id,
                     node=self.node.name,
                 ))
-                return (self.STATUS_SKIPPED, [_("Fault unavailable")])
+                return (self.STATUS_SKIPPED, self.SKIP_REASON_FAULT_UNAVAILABLE)
 
         if self.covered_by_autoskip_selector(autoskip_selector):
             io.debug(_(
                 "autoskip matches {item} on {node}"
             ).format(item=self.id, node=self.node.name))
-            return (self.STATUS_SKIPPED, [_("cmdline")])
+            return (self.STATUS_SKIPPED, self.SKIP_REASON_CMDLINE)
 
         if self._skip_with_soft_locks(my_soft_locks, other_peoples_soft_locks):
-            return (self.STATUS_SKIPPED, [_("soft locked")])
+            return (self.STATUS_SKIPPED, self.SKIP_REASON_SOFTLOCK)
 
         if interactive is False and self.attributes['interactive'] is True:
-            return (self.STATUS_SKIPPED, [_("interactive only")])
+            return (self.STATUS_SKIPPED, self.SKIP_REASON_INTERACTIVE_ONLY)
+
+        for item in self._precedes_items:
+            if item._triggers_preceding_items(interactive=interactive):
+                io.debug(_(
+                    "preceding item {item} on {node} has been triggered by {other_item}"
+                ).format(item=self.id, node=self.node.name, other_item=item.id))
+                self.has_been_triggered = True
+                break
+            else:
+                io.debug(_(
+                    "preceding item {item} on {node} has NOT been triggered by {other_item}"
+                ).format(item=self.id, node=self.node.name, other_item=item.id))
 
         if self.triggered and not self.has_been_triggered:
             io.debug(_("skipping {} because it wasn't triggered").format(self.id))
-            return (self.STATUS_SKIPPED, [_("no trigger")])
+            return (self.STATUS_SKIPPED, self.SKIP_REASON_NO_TRIGGER)
 
         if self.unless:
-            with io.job(_("  {node}  {bundle}  {item}  checking 'unless' condition...").format(
-                bundle=self.bundle.name,
+            with io.job(_("{node}  {bundle}  {item}  checking 'unless' condition").format(
+                bundle=bold(self.bundle.name),
                 item=self.id,
-                node=self.node.name,
+                node=bold(self.node.name),
             )):
                 unless_result = self.bundle.node.run(
                     self.unless,
@@ -83,7 +95,7 @@ class Action(Item):
                     name=self.name,
                     node=self.bundle.node.name,
                 ))
-                return (self.STATUS_SKIPPED, ["unless"])
+                return (self.STATUS_SKIPPED, self.SKIP_REASON_UNLESS)
 
         question_body = ""
         if self.attributes['data_stdin'] is not None:
@@ -114,7 +126,7 @@ class Action(Item):
                 ),
             )
         ):
-            return (self.STATUS_SKIPPED, [_("interactive")])
+            return (self.STATUS_SKIPPED, self.SKIP_REASON_INTERACTIVE)
         try:
             self.run()
             return (self.STATUS_ACTION_SUCCEEDED, None)
@@ -159,10 +171,10 @@ class Action(Item):
         else:
             data_stdin = None
 
-        with io.job(_("  {node}  {bundle}  {item}  running...").format(
-            bundle=self.bundle.name,
+        with io.job(_("{node}  {bundle}  {item}").format(
+            bundle=bold(self.bundle.name),
             item=self.id,
-            node=self.node.name,
+            node=bold(self.node.name),
         )):
             result = self.bundle.node.run(
                 self.attributes['command'],
