@@ -11,6 +11,10 @@ from subprocess import call
 from sys import exc_info
 from traceback import format_exception
 
+from jinja2 import Environment, FileSystemLoader
+from mako.lookup import TemplateLookup
+from mako.template import Template
+
 from bundlewrap.exceptions import BundleError, FaultUnavailable, TemplateError
 from bundlewrap.items import BUILTIN_ITEM_ATTRIBUTES, Item
 from bundlewrap.items.directories import validator_mode
@@ -30,14 +34,6 @@ def content_processor_base64(item):
 
 
 def content_processor_jinja2(item):
-    try:
-        from jinja2 import Environment, FileSystemLoader
-    except ImportError:
-        raise TemplateError(_(
-            "Unable to load Jinja2 (required to render {item}). "
-            "You probably have to install it using `pip install Jinja2`."
-        ).format(item=item.id))
-
     loader = FileSystemLoader(searchpath=[item.item_data_dir, item.item_dir])
     env = Environment(loader=loader)
 
@@ -80,8 +76,6 @@ def content_processor_jinja2(item):
 
 
 def content_processor_mako(item):
-    from mako.lookup import TemplateLookup
-    from mako.template import Template
     template = Template(
         item._template_content.encode('utf-8'),
         input_encoding='utf-8',
@@ -179,9 +173,9 @@ class File(Item):
         'context': None,
         'delete': False,
         'encoding': "utf-8",
-        'group': None,
-        'mode': None,
-        'owner': None,
+        'group': "root",
+        'mode': "0644",
+        'owner': "root",
         'source': None,
         'verify_with': None,
     }
@@ -258,6 +252,7 @@ class File(Item):
                 mode=self.attributes['mode'],
                 owner=self.attributes['owner'] or "",
                 group=self.attributes['group'] or "",
+                may_fail=True,
             )
 
     def _fix_mode(self, status):
@@ -351,14 +346,6 @@ class File(Item):
             }
 
     def display_dicts(self, cdict, sdict, keys):
-        if 'content' in keys:
-            del cdict['content_hash']
-            del sdict['content_hash']
-            cdict['content'] = self.content
-            sdict['content'] = get_remote_file_contents(self.node, self.name)
-        return (cdict, sdict)
-
-    def display_keys(self, cdict, sdict, keys):
         if (
             'content_hash' in keys and
             self.attributes['content_type'] not in ('base64', 'binary') and
@@ -367,7 +354,11 @@ class File(Item):
         ):
             keys.remove('content_hash')
             keys.append('content')
-        return keys
+            del cdict['content_hash']
+            del sdict['content_hash']
+            cdict['content'] = self.content
+            sdict['content'] = get_remote_file_contents(self.node, self.name)
+        return (cdict, sdict, keys)
 
     def patch_attributes(self, attributes):
         if (
@@ -381,6 +372,10 @@ class File(Item):
             attributes['context'] = {}
         if 'mode' in attributes and attributes['mode'] is not None:
             attributes['mode'] = str(attributes['mode']).zfill(4)
+        if 'group' not in attributes and self.node.os in self.node.OS_FAMILY_BSD:
+            # BSD doesn't have a root group, so we have to use a
+            # different default value here
+            attributes['group'] = 'wheel'
         return attributes
 
     def test(self):
