@@ -12,7 +12,6 @@ class AptPkg(Pkg):
     """
     A package installed by apt.
     """
-    BLOCK_CONCURRENT = ["pkg_apt"]
     BUNDLE_ATTRIBUTE_NAME = "pkg_apt"
     ITEM_TYPE_NAME = "pkg_apt"
     WHEN_CREATING_ATTRIBUTES = {
@@ -31,7 +30,8 @@ class AptPkg(Pkg):
             runlevel +
             "DEBIAN_FRONTEND=noninteractive "
             "apt-get -qy -o Dpkg::Options::=--force-confold --no-install-recommends "
-            "install {}".format(quote(self.name.replace("_", ":")))
+            "install {}".format(quote(self.name.replace("_", ":"))),
+            may_fail=True,
         )
 
     def pkg_installed(self):
