@@ -108,6 +108,14 @@ class User(Item):
     }
     ITEM_TYPE_NAME = "user"
 
+    @classmethod
+    def block_concurrent(cls, node_os, node_os_version):
+        # https://github.com/bundlewrap/bundlewrap/issues/367
+        if node_os == 'openbsd':
+            return [cls.ITEM_TYPE_NAME]
+        else:
+            return []
+
     def __repr__(self):
         return "<User name:{}>".format(self.name)
 
@@ -143,6 +151,20 @@ class User(Item):
             command += self.name
             self.node.run(command, may_fail=True)
 
+    def display_dicts(self, cdict, sdict, keys):
+        for attr_name, attr_display_name in _ATTRIBUTE_NAMES.items():
+            try:
+                keys.remove(attr_name)
+            except ValueError:
+                pass
+            else:
+                keys.append(attr_display_name)
+                cdict[attr_display_name] = cdict[attr_name]
+                sdict[attr_display_name] = sdict[attr_name]
+                del cdict[attr_name]
+                del sdict[attr_name]
+        return (cdict, sdict, keys)
+
     def get_auto_deps(self, items):
         deps = []
         groups = self.attributes['groups'] or []
@@ -236,6 +258,11 @@ class User(Item):
                     rounds=8,  # default rounds for OpenBSD accounts
                     salt=_DEFAULT_BCRYPT_SALT if salt is None else salt,
                 )
+            elif attributes.get('hash_method') == 'md5':
+                attributes['password_hash'] = hash_method.encrypt(
+                    force_text(attributes['password']),
+                    salt=_DEFAULT_SALT if salt is None else salt,
+                )
             else:
                 attributes['password_hash'] = hash_method.encrypt(
                     force_text(attributes['password']),
