@@ -11,8 +11,16 @@ from time import time
 
 from .exceptions import NodeLockedException
 from .utils import cached_property, tempfile
-from .utils.text import blue, bold, mark_for_translation as _, red, wrap_question
-from .utils.time import format_duration, format_timestamp, parse_duration
+from .utils.text import (
+    blue,
+    bold,
+    format_duration,
+    format_timestamp,
+    mark_for_translation as _,
+    parse_duration,
+    red,
+    wrap_question,
+)
 from .utils.ui import io
 
 
@@ -38,7 +46,7 @@ class NodeLock(object):
     def __enter__(self):
         with tempfile() as local_path:
             if not self.ignore:
-                with io.job(_("  {node}  checking hard lock status...").format(node=self.node.name)):
+                with io.job(_("{node}  checking hard lock status").format(node=bold(self.node.name))):
                     result = self.node.run("mkdir " + quote(HARD_LOCK_PATH), may_fail=True)
                     if result.return_code != 0:
                         self.node.download(HARD_LOCK_FILE, local_path)
@@ -80,7 +88,7 @@ class NodeLock(object):
                         else:
                             raise NodeLockedException(info)
 
-            with io.job(_("  {node}  uploading lock file...").format(node=self.node.name)):
+            with io.job(_("{node}  uploading lock file").format(node=bold(self.node.name))):
                 if self.ignore:
                     self.node.run("mkdir -p " + quote(HARD_LOCK_PATH))
                 with open(local_path, 'w') as f:
@@ -93,7 +101,7 @@ class NodeLock(object):
         return self
 
     def __exit__(self, type, value, traceback):
-        with io.job(_("  {node}  removing hard lock...").format(node=self.node.name)):
+        with io.job(_("{node}  removing hard lock").format(node=bold(self.node.name))):
             result = self.node.run("rm -R {}".format(quote(HARD_LOCK_PATH)), may_fail=True)
 
         if result.return_code != 0:
@@ -168,7 +176,7 @@ def softlock_add(node, lock_id, comment="", expiry="8h", item_selectors=None):
 
 
 def softlock_list(node):
-    with io.job(_("  {}  checking soft locks...").format(node.name)):
+    with io.job(_("{}  checking soft locks").format(bold(node.name))):
         cat = node.run("cat {}".format(SOFT_LOCK_FILE.format(id="*")), may_fail=True)
         if cat.return_code != 0:
             return []
