@@ -4,7 +4,6 @@ from __future__ import unicode_literals
 from datetime import datetime, timedelta
 from hashlib import md5
 from os import environ
-from sys import exit
 from threading import Lock
 
 from . import operations
@@ -16,22 +15,30 @@ from .deps import (
 )
 from .exceptions import (
     DontCache,
-    FaultUnavailable,
     ItemDependencyLoop,
     NodeLockedException,
     NoSuchBundle,
     RepositoryError,
 )
 from .group import GROUP_ATTR_DEFAULTS
-from .itemqueue import ItemQueue, ItemTestQueue
+from .itemqueue import ItemQueue
 from .items import Item
 from .lock import NodeLock
-from .metadata import check_for_unsolvable_metadata_key_conflicts, hash_metadata
+from .metadata import hash_metadata
 from .utils import cached_property, names
-from .utils.statedict import hash_statedict
-from .utils.text import blue, bold, cyan, green, red, validate_name, yellow
-from .utils.text import force_text, mark_for_translation as _
-from .utils.time import format_duration
+from .utils.dicts import hash_statedict
+from .utils.text import (
+    blue,
+    bold,
+    cyan,
+    force_text,
+    format_duration,
+    green,
+    mark_for_translation as _,
+    red,
+    validate_name,
+    yellow,
+)
 from .utils.ui import io
 
 
@@ -45,10 +52,10 @@ class ApplyResult(object):
         self.fixed = 0
         self.skipped = 0
         self.failed = 0
-        self.profiling_info = []
+        self.total = 0
 
-        for item_id, result, time_elapsed in item_results:
-            self.profiling_info.append((time_elapsed, item_id))
+        for item_id, result, duration in item_results:
+            self.total += 1
             if result == Item.STATUS_ACTION_SUCCEEDED:
                 self.correct += 1
             elif result == Item.STATUS_OK:
@@ -64,9 +71,6 @@ class ApplyResult(object):
                     "can't make sense of results for {} on {}: {}"
                 ).format(item_id, self.node_name, result))
 
-        self.profiling_info.sort()
-        self.profiling_info.reverse()
-
         self.start = None
         self.end = None
 
@@ -97,14 +101,21 @@ def format_node_result(result):
     return ", ".join(output)
 
 
-def handle_apply_result(node, item, status_code, interactive, changes=None):
+def handle_apply_result(node, item, status_code, interactive, details=None):
+    if status_code == Item.STATUS_SKIPPED and details in (
+        Item.SKIP_REASON_CMDLINE,
+        Item.SKIP_REASON_NO_TRIGGER,
+        Item.SKIP_REASON_UNLESS,
+    ):
+        # skipped for "unless" or "not triggered", don't output those
+        return
     formatted_result = format_item_result(
         status_code,
         node.name,
         item.bundle.name if item.bundle else "",  # dummy items don't have bundles
         item.id,
         interactive=interactive,
-        changes=changes,
+        details=details,
     )
     if formatted_result is not None:
         if status_code == Item.STATUS_FAILED:
@@ -120,10 +131,12 @@ def apply_items(
     other_peoples_soft_locks=(),
     workers=1,
     interactive=False,
-    profiling=False,
 ):
-    with io.job(_("  {node}  processing dependencies...").format(node=node.name)):
-        item_queue = ItemQueue(node.items)
+    item_queue = ItemQueue(node.items, node.name, node.os, node.os_version)
+    # the item queue might contain new generated items (canned actions,
+    # dummy items); adjust progress total accordingly
+    extra_items = len(item_queue.all_items) - len(node.items)
+    io.progress_increase_total(increment=extra_items)
 
     results = []
 
@@ -131,18 +144,7 @@ def apply_items(
         return bool(item_queue.items_without_deps)
 
     def next_task():
-        item, skipped_items = item_queue.pop()
-        for skipped_item in skipped_items:
-            io.progress_advance()
-            handle_apply_result(
-                node,
-                skipped_item,
-                Item.STATUS_SKIPPED,
-                interactive,
-                changes=[_("no pre-trigger")],
-            )
-            results.append((skipped_item.id, Item.STATUS_SKIPPED, timedelta(0)))
-
+        item = item_queue.pop()
         return {
             'task_id': "{}:{}".format(node.name, item.id),
             'target': item.apply,
@@ -158,7 +160,7 @@ def apply_items(
         item_id = task_id.split(":", 1)[1]
         item = find_item(item_id, item_queue.pending_items)
 
-        status_code, changes = return_value
+        status_code, details = return_value
 
         if status_code == Item.STATUS_FAILED:
             for skipped_item in item_queue.item_failed(item):
@@ -167,7 +169,7 @@ def apply_items(
                     skipped_item,
                     Item.STATUS_SKIPPED,
                     interactive,
-                    changes=[_("dep failed")],
+                    details=Item.SKIP_REASON_DEP_FAILED,
                 )
                 results.append((skipped_item.id, Item.STATUS_SKIPPED, timedelta(0)))
         elif status_code in (Item.STATUS_FIXED, Item.STATUS_ACTION_SUCCEEDED):
@@ -176,18 +178,18 @@ def apply_items(
             item_queue.item_ok(item)
         elif status_code == Item.STATUS_SKIPPED:
             for skipped_item in item_queue.item_skipped(item):
-                skipped_reason = [_("dep skipped")]
+                skip_reason = Item.SKIP_REASON_DEP_FAILED
                 for lock in other_peoples_soft_locks:
                     for selector in lock['items']:
                         if skipped_item.covered_by_autoskip_selector(selector):
-                            skipped_reason = [_("soft locked")]
+                            skip_reason = Item.SKIP_REASON_SOFTLOCK
                             break
                 handle_apply_result(
                     node,
                     skipped_item,
                     Item.STATUS_SKIPPED,
                     interactive,
-                    changes=skipped_reason,
+                    details=skip_reason,
                 )
                 results.append((skipped_item.id, Item.STATUS_SKIPPED, timedelta(0)))
         else:
@@ -198,9 +200,9 @@ def apply_items(
                 ),
             ))
 
-        handle_apply_result(node, item, status_code, interactive, changes=changes)
+        handle_apply_result(node, item, status_code, interactive, details=details)
+        io.progress_advance()
         if not isinstance(item, DummyItem):
-            io.progress_advance()
             results.append((item.id, status_code, duration))
 
     worker_pool = WorkerPool(
@@ -264,19 +266,21 @@ def _flatten_group_hierarchy(groups):
     return order
 
 
-def format_item_result(result, node, bundle, item, interactive=False, changes=None):
-    if changes is True:
-        changes_text = "({})".format(_("create"))
-    elif changes is False:
-        changes_text = "({})".format(_("remove"))
-    elif changes is None:
-        changes_text = ""
+def format_item_result(result, node, bundle, item, interactive=False, details=None):
+    if details is True:
+        details_text = "({})".format(_("create"))
+    elif details is False:
+        details_text = "({})".format(_("remove"))
+    elif details is None:
+        details_text = ""
+    elif result == Item.STATUS_SKIPPED:
+        details_text = "({})".format(Item.SKIP_REASON_DESC[details])
     else:
-        changes_text = "({})".format(", ".join(sorted(changes)))
+        details_text = "({})".format(", ".join(sorted(details)))
     if result == Item.STATUS_FAILED:
-        return "{x} {node}  {bundle}  {item} {status} {changes}".format(
+        return "{x} {node}  {bundle}  {item} {status} {details}".format(
             bundle=bold(bundle),
-            changes=changes_text,
+            details=details_text,
             item=item,
             node=bold(node),
             status=red(_("failed")),
@@ -291,18 +295,18 @@ def format_item_result(result, node, bundle, item, interactive=False, changes=No
             x=bold(green("✓")),
         )
     elif result == Item.STATUS_SKIPPED:
-        return "{x} {node}  {bundle}  {item} {status} {changes}".format(
+        return "{x} {node}  {bundle}  {item} {status} {details}".format(
             bundle=bold(bundle),
-            changes=changes_text,
+            details=details_text,
             item=item,
             node=bold(node),
             x=bold(yellow("»")),
             status=yellow(_("skipped")),
         )
     elif result == Item.STATUS_FIXED:
-        return "{x} {node}  {bundle}  {item} {status} {changes}".format(
+        return "{x} {node}  {bundle}  {item} {status} {details}".format(
             bundle=bold(bundle),
-            changes=changes_text,
+            details=details_text,
             item=item,
             node=bold(node),
             x=bold(green("✓")),
@@ -333,6 +337,7 @@ class Node(object):
         'amazonlinux',
         'arch',
         'opensuse',
+        'openwrt',
         'gentoo',
         'linux',
     ) + \
@@ -380,7 +385,7 @@ class Node(object):
 
     @cached_property
     def bundles(self):
-        with io.job(_("  {node}  loading bundles...").format(node=self.name)):
+        with io.job(_("{node}  loading bundles").format(node=bold(self.name))):
             added_bundles = []
             found_bundles = []
             for group in self.groups:
@@ -427,6 +432,7 @@ class Node(object):
         return hash_statedict(sorted(names(self.groups)))
 
     @cached_property
+    @io.job_wrapper(_("{}  determining groups").format(bold("{0.name}")))
     def groups(self):
         _groups = set(self.repo._static_groups_for_node(self))
         # lock to avoid infinite recursion when .members_add/remove
@@ -533,12 +539,6 @@ class Node(object):
     def magic_number(self):
         return int(md5(self.name.encode('UTF-8')).hexdigest(), 16)
 
-    @property
-    def _static_items(self):
-        for bundle in self.bundles:
-            for item in bundle._static_items:
-                yield item
-
     def apply(
         self,
         autoskip_selector="",
@@ -546,7 +546,6 @@ class Node(object):
         force=False,
         skip_list=tuple(),
         workers=4,
-        profiling=False,
     ):
         if not list(self.items):
             io.stdout(_("{x} {node}  has no items").format(
@@ -592,7 +591,6 @@ class Node(object):
                     other_peoples_soft_locks=lock.other_peoples_soft_locks,
                     workers=workers,
                     interactive=interactive,
-                    profiling=profiling,
                 )
         except NodeLockedException as e:
             if not interactive:
@@ -656,6 +654,10 @@ class Node(object):
         else:
             return self.repo._metadata_for_node(self.name, partial=False)
 
+    @property
+    def metadata_blame(self):
+        return self.repo._metadata_for_node(self.name, partial=False, blame=True)
+
     def metadata_hash(self):
         return hash_metadata(self.metadata)
 
@@ -727,21 +729,7 @@ class Node(object):
             wrapper_outer=self.cmd_wrapper_outer,
         )
 
-    def test(self, ignore_missing_faults=False, workers=4):
-        with io.job(_("  {node}  checking for metadata collisions...").format(node=self.name)):
-            check_for_unsolvable_metadata_key_conflicts(self)
-        io.stdout(_("{x} {node}  has no metadata collisions").format(
-            x=green("✓"),
-            node=bold(self.name),
-        ))
-        if self.items:
-            test_items(self, ignore_missing_faults=ignore_missing_faults, workers=workers)
-        else:
-            io.stdout(_("{x} {node}  has no items").format(node=bold(self.name), x=yellow("!")))
-
-        self.repo.hooks.test_node(self.repo, self)
-
-    def upload(self, local_path, remote_path, mode=None, owner="", group=""):
+    def upload(self, local_path, remote_path, mode=None, owner="", group="", may_fail=False):
         return operations.upload(
             self.hostname,
             local_path,
@@ -750,6 +738,7 @@ class Node(object):
             group=group,
             mode=mode,
             owner=owner,
+            ignore_failure=may_fail,
             wrapper_inner=self.cmd_wrapper_inner,
             wrapper_outer=self.cmd_wrapper_outer,
         )
@@ -814,75 +803,6 @@ for attr, default in GROUP_ATTR_DEFAULTS.items():
     setattr(Node, attr, build_attr_property(attr, default))
 
 
-def test_items(node, ignore_missing_faults=False, workers=1):
-    item_queue = ItemTestQueue(node.items)
-
-    def tasks_available():
-        return bool(item_queue.items_without_deps)
-
-    def next_task():
-        try:
-            # Get the next non-DummyItem in the queue.
-            while True:
-                item = item_queue.pop()
-                if not isinstance(item, DummyItem):
-                    break
-        except IndexError:  # no more items available right now
-            return None
-        else:
-            return {
-                'task_id': item.node.name + ":" + item.bundle.name + ":" + item.id,
-                'target': item._test,
-            }
-
-    def handle_result(task_id, return_value, duration):
-        node_name, bundle_name, item_id = task_id.split(":", 2)
-        if item_id.count(":") < 2:
-            # don't count canned actions
-            io.progress_advance()
-        io.stdout("{x} {node}  {bundle}  {item}".format(
-            bundle=bold(bundle_name),
-            item=item_id,
-            node=bold(node_name),
-            x=green("✓"),
-        ))
-
-    def handle_exception(task_id, exception, traceback):
-        io.progress_advance()
-        node_name, bundle_name, item_id = task_id.split(":", 2)
-        if ignore_missing_faults and isinstance(exception, FaultUnavailable):
-            io.stderr(_("{x} {node}  {bundle}  {item}  ({msg})").format(
-                bundle=bold(bundle_name),
-                item=item_id,
-                msg=yellow(_("Fault unavailable")),
-                node=bold(node_name),
-                x=yellow("»"),
-            ))
-        else:
-            io.stderr("{x} {node}  {bundle}  {item}".format(
-                bundle=bold(bundle_name),
-                item=item_id,
-                node=bold(node_name),
-                x=red("!"),
-            ))
-            io.stderr(traceback)
-            io.stderr("{}: {}".format(type(exception), str(exception)))
-            exit(1)
-
-    worker_pool = WorkerPool(
-        tasks_available,
-        next_task,
-        handle_result=handle_result,
-        handle_exception=handle_exception,
-        pool_id="test_{}".format(node.name),
-        workers=workers,
-    )
-    worker_pool.run()
-
-    if item_queue.items_with_deps:
-        raise ItemDependencyLoop(item_queue.items_with_deps)
-
-
 def verify_items(node, show_all=False, workers=1):
     items = []
     for item in node.items:
@@ -927,14 +847,14 @@ def verify_items(node, show_all=False, workers=1):
         node_name, bundle_name, item_id = task_id.split(":", 2)
         if not unless_result and not item_status.correct:
             if item_status.must_be_created:
-                changes_text = _("create")
+                details_text = _("create")
             elif item_status.must_be_deleted:
-                changes_text = _("remove")
+                details_text = _("remove")
             else:
-                changes_text = ", ".join(sorted(item_status.keys_to_fix))
-            io.stderr("{x} {node}  {bundle}  {item} ({changes})".format(
+                details_text = ", ".join(sorted(item_status.display_keys_to_fix))
+            io.stderr("{x} {node}  {bundle}  {item} ({details})".format(
                 bundle=bold(bundle_name),
-                changes=changes_text,
+                details=details_text,
                 item=item_id,
                 node=bold(node_name),
                 x=red("✘"),
