@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
+from datetime import datetime
 from pipes import quote
 from select import select
 from shlex import split
@@ -73,6 +74,7 @@ def download(
 
 class RunResult(object):
     def __init__(self):
+        self.duration = None
         self.return_code = None
         self.stderr = None
         self.stdout = None
@@ -111,6 +113,7 @@ def run_local(
 
     cmd_id = randstr(length=4).upper()
     io.debug("running command with ID {}: {}".format(cmd_id, " ".join(command)))
+    start = datetime.utcnow()
 
     # Launch the child process. It's important that SSH gets a dummy
     # stdin, i.e. it must *not* read from the terminal. Otherwise, it
@@ -181,6 +184,7 @@ def run_local(
     ))
 
     result = RunResult()
+    result.duration = datetime.utcnow() - start
     result.stdout = stdout_lb.record.getvalue()
     result.stderr = stderr_lb.record.getvalue()
     result.return_code = child_process.returncode
@@ -242,6 +246,7 @@ def upload(
     group="",
     mode=None,
     owner="",
+    ignore_failure=False,
     wrapper_inner="{}",
     wrapper_outer="{}",
 ):
@@ -256,12 +261,18 @@ def upload(
         "scp",
         "-o",
         "StrictHostKeyChecking=no" if add_host_keys else "StrictHostKeyChecking=yes",
-        local_path,
-        "{}:{}".format(hostname, temp_filename),
     ]
+    extra_args = environ.get("BW_SCP_ARGS", environ.get("BW_SSH_ARGS", "")).strip()
+    if extra_args:
+        scp_command.extend(split(extra_args))
+    scp_command.append(local_path)
+    scp_command.append("{}:{}".format(hostname, temp_filename))
+
     scp_process = run_local(scp_command)
 
     if scp_process.return_code != 0:
+        if ignore_failure:
+            return False
         raise RemoteException(_(
             "Upload to {host} failed for {failed}:\n\n{result}\n\n"
         ).format(
@@ -273,7 +284,7 @@ def upload(
     if owner or group:
         if group:
             group = ":" + quote(group)
-        run(
+        result = run(
             hostname,
             "chown {}{} {}".format(
                 quote(owner),
@@ -281,29 +292,37 @@ def upload(
                 quote(temp_filename),
             ),
             add_host_keys=add_host_keys,
+            ignore_failure=ignore_failure,
             wrapper_inner=wrapper_inner,
             wrapper_outer=wrapper_outer,
         )
+        if result.return_code != 0:
+            return False
 
     if mode:
-        run(
+        result = run(
             hostname,
             "chmod {} {}".format(
                 mode,
                 quote(temp_filename),
             ),
             add_host_keys=add_host_keys,
+            ignore_failure=ignore_failure,
             wrapper_inner=wrapper_inner,
             wrapper_outer=wrapper_outer,
         )
+        if result.return_code != 0:
+            return False
 
-    run(
+    result = run(
         hostname,
         "mv -f {} {}".format(
             quote(temp_filename),
             quote(remote_path),
         ),
         add_host_keys=add_host_keys,
+        ignore_failure=ignore_failure,
         wrapper_inner=wrapper_inner,
         wrapper_outer=wrapper_outer,
     )
+    return result.return_code == 0
