@@ -12,7 +12,6 @@ from pkg_resources import DistributionNotFound, require, VersionConflict
 from . import items, utils, VERSION_STRING
 from .bundle import FILENAME_BUNDLE
 from .exceptions import (
-    BundleError,
     NoSuchGroup,
     NoSuchNode,
     NoSuchRepository,
@@ -20,13 +19,20 @@ from .exceptions import (
     RepositoryError,
 )
 from .group import Group
-from .metadata import deepcopy_metadata
+from .metadata import (
+    blame_changed_paths,
+    check_metadata_processor_result,
+    deepcopy_metadata,
+    DEFAULTS,
+    DONE,
+    OVERWRITE,
+)
 from .node import _flatten_group_hierarchy, Node
 from .secrets import FILENAME_SECRETS, generate_initial_secrets_cfg, SecretProxy
-from .utils import cached_property, merge_dict, names
+from .utils import cached_property, names
 from .utils.scm import get_rev
-from .utils.statedict import hash_statedict
-from .utils.text import mark_for_translation as _, red, validate_name
+from .utils.dicts import hash_statedict, merge_dict
+from .utils.text import bold, mark_for_translation as _, red, validate_name
 from .utils.ui import io, QUIT_EVENT
 
 DIRNAME_BUNDLES = "bundles"
@@ -90,7 +96,6 @@ nodes = {
     FILENAME_REQUIREMENTS: "bundlewrap>={}\n".format(VERSION_STRING),
     FILENAME_SECRETS: generate_initial_secrets_cfg,
 }
-META_PROC_MAX_ITER = 1000  # maximum iterations for metadata processors
 
 
 def groups_from_file(filepath, libs, repo_path, vault):
@@ -252,6 +257,7 @@ class Repository(object):
         self.bundle_names = []
         self.group_dict = {}
         self.node_dict = {}
+        self._node_metadata_blame = {}
         self._node_metadata_complete = {}
         self._node_metadata_partial = {}
         self._node_metadata_static_complete = set()
@@ -420,7 +426,7 @@ class Repository(object):
         """
         return self.nodes_in_all_groups([group_name])
 
-    def _metadata_for_node(self, node_name, partial=False):
+    def _metadata_for_node(self, node_name, partial=False, blame=False):
         """
         Returns full or partial metadata for this node.
 
@@ -448,7 +454,7 @@ class Repository(object):
                 pass
 
             self._node_metadata_partial[node_name] = {}
-            self._build_node_metadata()
+            self._build_node_metadata(blame=blame)
 
             # now that we have completed all metadata for this
             # node and all related nodes, copy that data over
@@ -459,68 +465,96 @@ class Repository(object):
             self._node_metadata_partial = {}
             self._node_metadata_static_complete = set()
 
-            return self._node_metadata_complete[node_name]
+            if blame:
+                return self._node_metadata_blame[node_name]
+            else:
+                return self._node_metadata_complete[node_name]
 
-    def _build_node_metadata(self):
+    def _build_node_metadata(self, blame=False):
         """
         Builds complete metadata for all nodes that appear in
         self._node_metadata_partial.keys().
         """
-        iterations = {}
         # these processors have indicated that they do not need to be run again
         blacklisted_metaprocs = set()
-        while (
-            not iterations or max(iterations.values()) <= META_PROC_MAX_ITER
-        ) and not QUIT_EVENT.is_set():
+        while not QUIT_EVENT.is_set():
             # First, get the static metadata out of the way
             for node_name in list(self._node_metadata_partial):
                 if QUIT_EVENT.is_set():
                     break
                 node = self.get_node(node_name)
+                node_blame = self._node_metadata_blame.setdefault(node_name, {})
                 # check if static metadata for this node is already done
                 if node_name in self._node_metadata_static_complete:
                     continue
                 else:
                     self._node_metadata_static_complete.add(node_name)
 
-                with io.job(_("  {node}  building group metadata...").format(node=node.name)):
+                with io.job(_("{node}  building group metadata").format(node=bold(node.name))):
                     group_order = _flatten_group_hierarchy(node.groups)
                     for group_name in group_order:
-                        self._node_metadata_partial[node.name] = merge_dict(
+                        new_metadata = merge_dict(
                             self._node_metadata_partial[node.name],
                             self.get_group(group_name).metadata,
                         )
+                        if blame:
+                            blame_changed_paths(
+                                self._node_metadata_partial[node.name],
+                                new_metadata,
+                                node_blame,
+                                "group:{}".format(group_name),
+                            )
+                        self._node_metadata_partial[node.name] = new_metadata
 
-                with io.job(_("  {node}  merging node metadata...").format(node=node.name)):
-                    self._node_metadata_partial[node.name] = merge_dict(
+                with io.job(_("{node}  merging node metadata").format(node=bold(node.name))):
+                    # deepcopy_metadata is important here because up to this point
+                    # different nodes from the same group might still share objects
+                    # nested deeply in their metadata. This becomes a problem if we
+                    # start messing with these objects in metadata processors. Every
+                    # time we would edit one of these objects, the changes would be
+                    # shared amongst multiple nodes.
+                    new_metadata = deepcopy_metadata(merge_dict(
                         self._node_metadata_partial[node.name],
                         node._node_metadata,
-                    )
+                    ))
+                    if blame:
+                        blame_changed_paths(
+                            self._node_metadata_partial[node.name],
+                            new_metadata,
+                            node_blame,
+                            "node:{}".format(node.name),
+                        )
+                    self._node_metadata_partial[node.name] = new_metadata
 
             # Now for the interesting part: We run all metadata processors
-            # in sequence until none of them return changed metadata.
-            modified = False
+            # until none of them return DONE anymore (indicating that they're
+            # just waiting for another metaproc to maybe insert new data,
+            # which isn't happening if none return DONE)
+            metaproc_returned_DONE = False
             for node_name in list(self._node_metadata_partial):
                 if QUIT_EVENT.is_set():
                     break
                 node = self.get_node(node_name)
-                with io.job(_("  {node}  running metadata processors...").format(node=node.name)):
+                node_blame = self._node_metadata_blame[node_name]
+                with io.job(_("{node}  running metadata processors").format(node=bold(node.name))):
                     for metadata_processor_name, metadata_processor in node.metadata_processors:
                         if (node_name, metadata_processor_name) in blacklisted_metaprocs:
                             continue
-                        iterations.setdefault((node.name, metadata_processor_name), 1)
                         io.debug(_(
-                            "running metadata processor {metaproc} for node {node}, "
-                            "iteration #{i}"
+                            "running metadata processor {metaproc} for node {node}"
                         ).format(
                             metaproc=metadata_processor_name,
                             node=node.name,
-                            i=iterations[(node.name, metadata_processor_name)],
                         ))
+                        if blame:
+                            # We need to deepcopy here because otherwise we have no chance of
+                            # figuring out what changed...
+                            input_metadata = deepcopy_metadata(self._node_metadata_partial[node.name])
+                        else:
+                            # ...but we can't always do it for performance reasons.
+                            input_metadata = self._node_metadata_partial[node.name]
                         try:
-                            processed = metadata_processor(
-                                deepcopy_metadata(self._node_metadata_partial[node.name]),
-                            )
+                            processed = metadata_processor(input_metadata)
                         except Exception as exc:
                             io.stderr(_(
                                 "{x} Exception while executing metadata processor "
@@ -531,37 +565,55 @@ class Repository(object):
                                 node=node.name,
                             ))
                             raise exc
-                        iterations[(node.name, metadata_processor_name)] += 1
-                        if isinstance(processed, tuple) and len(processed) == 2:
-                            if processed[1] is True:
-                                io.debug(_(
-                                    "metadata processor {metaproc} for node {node} "
-                                    "has indicated that it need not be run again"
-                                ).format(
-                                    metaproc=metadata_processor_name,
-                                    node=node.name,
-                                ))
-                                blacklisted_metaprocs.add((node_name, metadata_processor_name))
-                            processed = processed[0]
-                        if not isinstance(processed, dict):
-                            raise ValueError(_(
-                                "metadata processor {metaproc} for node {node} did not return "
-                                "a dictionary or tuple of (dict, bool)"
-                            ).format(
-                                metaproc=metadata_processor_name,
-                                node=node.name,
-                            ))
-                        if processed != self._node_metadata_partial[node.name]:
+                        processed_dict, options = check_metadata_processor_result(
+                            processed,
+                            node.name,
+                            metadata_processor_name,
+                        )
+                        if DONE in options:
                             io.debug(_(
-                                "metadata processor {metaproc} for node {node} changed metadata, "
-                                "rerunning all metadata processors for this node"
+                                "metadata processor {metaproc} for node {node} "
+                                "has indicated that it need NOT be run again"
                             ).format(
                                 metaproc=metadata_processor_name,
                                 node=node.name,
                             ))
-                            self._node_metadata_partial[node.name] = processed
-                            modified = True
-            if not modified:
+                            blacklisted_metaprocs.add((node_name, metadata_processor_name))
+                            metaproc_returned_DONE = True
+                        else:
+                            io.debug(_(
+                                "metadata processor {metaproc} for node {node} "
+                                "has indicated that it must be run again"
+                            ).format(
+                                metaproc=metadata_processor_name,
+                                node=node.name,
+                            ))
+
+                        blame_defaults = False
+                        if DEFAULTS in options:
+                            processed_dict = merge_dict(
+                                processed_dict,
+                                self._node_metadata_partial[node.name],
+                            )
+                            blame_defaults = True
+                        elif OVERWRITE in options:
+                            processed_dict = merge_dict(
+                                self._node_metadata_partial[node.name],
+                                processed_dict,
+                            )
+
+                        if blame:
+                            blame_changed_paths(
+                                self._node_metadata_partial[node.name],
+                                processed_dict,
+                                node_blame,
+                                "metadata_processor:{}".format(metadata_processor_name),
+                                defaults=blame_defaults,
+                            )
+
+                        self._node_metadata_partial[node.name] = processed_dict
+
+            if not metaproc_returned_DONE:
                 if self._node_metadata_static_complete != set(self._node_metadata_partial.keys()):
                     # During metadata processor execution, partial metadata may
                     # have been requested for nodes we did not previously
@@ -574,25 +626,6 @@ class Repository(object):
                 else:
                     break
 
-        for culprit, number_of_iterations in iterations.items():
-            if number_of_iterations >= META_PROC_MAX_ITER:
-                node, metadata_processor = culprit
-                raise BundleError(_(
-                    "Metadata processor '{proc}' stopped after too many iterations "
-                    "({max_iter}) for node '{node}' to prevent infinite loop. "
-                    "This usually means one of two things: "
-                    "1) You have two metadata processors that keep overwriting each other's "
-                    "data or 2) You have a single metadata processor that keeps changing its own "
-                    "data. "
-                    "To fix this, use `bw --debug metadata {node}` and look for repeated messages "
-                    "indicating that the same metadata processor keeps changing metadata. Then "
-                    "rewrite that metadata processor to eventually stop changing metadata.".format(
-                        max_iter=META_PROC_MAX_ITER,
-                        node=node,
-                        proc=metadata_processor,
-                    ),
-                ))
-
     def metadata_hash(self):
         repo_dict = {}
         for node in self.nodes:
