@@ -75,6 +75,7 @@ class SecretProxy(object):
     def __init__(self, repo):
         self.repo = repo
         self.keys = self._load_keys()
+        self._call_log = {}
 
     def _decrypt(self, cryptotext=None, key='encrypt'):
         """
@@ -308,27 +309,11 @@ class SecretProxy(object):
             f.write(fernet.encrypt(plaintext))
         return target_file
 
-    def _format(self, format_str=None, faults=None):
-        return format_str.format(*[fault.value for fault in faults])
-
-    def format(self, format_str, *faults):
-        """
-        Returns a Fault for a string formatted with the given Faults,
-        e.g.:
-
-            vault.format("password: {}", vault.password_for("something"))
-
-        DEPRECATED, remove in 3.0, use Fault.format_into instead.
-        """
-        return Fault(
-            self._format,
-            format_str=format_str,
-            faults=faults,
-        )
-
     def human_password_for(
         self, identifier, digits=2, key='generate', per_word=3, words=4,
     ):
+        self._call_log.setdefault(identifier, 0)
+        self._call_log[identifier] += 1
         return Fault(
             self._generate_human_password,
             identifier=identifier,
@@ -339,6 +324,8 @@ class SecretProxy(object):
         )
 
     def password_for(self, identifier, key='generate', length=32, symbols=False):
+        self._call_log.setdefault(identifier, 0)
+        self._call_log[identifier] += 1
         return Fault(
             self._generate_password,
             identifier=identifier,
