@@ -14,12 +14,21 @@ import struct
 from subprocess import PIPE, Popen
 import sys
 import termios
+from time import time
 from threading import Event, Lock, Thread
 
 from . import STDERR_WRITER, STDOUT_WRITER
 from .table import render_table, ROW_SEPARATOR
-from .text import ansi_clean, blue, bold, inverse, mark_for_translation as _
-from .time import format_duration
+from .text import (
+    HIDE_CURSOR,
+    SHOW_CURSOR,
+    ansi_clean,
+    blue,
+    bold,
+    format_duration,
+    mark_for_translation as _,
+)
+
 
 INFO_EVENT = Event()
 QUIT_EVENT = Event()
@@ -53,7 +62,7 @@ def add_debug_timestamp(f):
 def capture_for_debug_logfile(f):
     @wraps(f)
     def wrapped(self, msg, **kwargs):
-        if self.debug_log_file:
+        if self.debug_log_file and self._active:
             self.debug_log_file.write(
                 datetime.now().strftime("[%Y-%m-%d %H:%M:%S.%f] ") +
                 ansi_clean(msg).rstrip("\n") + "\n"
@@ -97,6 +106,12 @@ def sigquit_handler(*args, **kwargs):
     INFO_EVENT.set()
 
 
+def spinner():
+    while True:
+        for c in "⠋⠙⠹⠸⠼⠴⠦⠧⠇⠏":
+            yield c
+
+
 def term_width():
     if not TTY:
         return 0
@@ -161,6 +176,9 @@ class IOManager(object):
         self.progress = 0
         self.progress_start = None
         self.progress_total = 0
+        self._spinner = spinner()
+        self._last_spinner_character = next(self._spinner)
+        self._last_spinner_update = 0
         self._signal_handler_thread = Thread(
             target=self._signal_handler_thread_body,
         )
@@ -170,6 +188,8 @@ class IOManager(object):
         # the thread once the soft shutdown has completed
         self._signal_handler_thread.daemon = True
         self._child_pids = []
+        self._status_line_present = False
+        self._waiting_for_input = False
 
     def activate(self):
         self._active = True
@@ -184,17 +204,20 @@ class IOManager(object):
         self._signal_handler_thread.start()
         signal(SIGINT, sigint_handler)
         signal(SIGQUIT, sigquit_handler)
+        if TTY:
+            write_to_stream(STDOUT_WRITER, HIDE_CURSOR)
 
     def ask(self, question, default, epilogue=None, input_handler=DrainableStdin()):
         assert self._active
         answers = _("[Y/n]") if default else _("[y/N]")
         question = question + " " + answers + " "
+        self._waiting_for_input = True
         with self.lock:
             if QUIT_EVENT.is_set():
                 sys.exit(0)
             self._clear_last_job()
             while True:
-                write_to_stream(STDOUT_WRITER, "\a" + question)
+                write_to_stream(STDOUT_WRITER, "\a" + question + SHOW_CURSOR)
 
                 input_handler.drain()
                 answer = input_handler.get_input()
@@ -219,11 +242,14 @@ class IOManager(object):
                 )
             if epilogue:
                 write_to_stream(STDOUT_WRITER, epilogue + "\n")
-            self._write_current_job()
+            write_to_stream(STDOUT_WRITER, HIDE_CURSOR)
+        self._waiting_for_input = False
         return answer
 
     def deactivate(self):
         self._active = False
+        if TTY:
+            write_to_stream(STDOUT_WRITER, SHOW_CURSOR)
         signal(SIGINT, SIG_DFL)
         signal(SIGQUIT, SIG_DFL)
         self._signal_handler_thread.join()
@@ -243,10 +269,9 @@ class IOManager(object):
         if not self._active:
             return
         with self.lock:
-            if TTY:
-                self._clear_last_job()
-                write_to_stream(STDOUT_WRITER, inverse("{} ".format(msg)[:term_width() - 1]))
+            self._clear_last_job()
             self.jobs.append(msg)
+            self._write_current_job()
 
     def job_del(self, msg):
         if not self._active:
@@ -260,6 +285,10 @@ class IOManager(object):
         with self.lock:
             self.progress += increment
 
+    def progress_increase_total(self, increment=1):
+        with self.lock:
+            self.progress_total += increment
+
     def progress_set_total(self, total):
         self.progress = 0
         self.progress_start = datetime.utcnow()
@@ -322,13 +351,26 @@ class IOManager(object):
         finally:
             self.job_del(job_text)
 
+    def job_wrapper(self, job_text):
+        def outer_wrapper(wrapped_function):
+            def inner_wrapper(*args, **kwargs):
+                with self.job(job_text.format(*args, **kwargs)):
+                    return wrapped_function(*args, **kwargs)
+            return inner_wrapper
+        return outer_wrapper
+
     def _clear_last_job(self):
-        if self.jobs and TTY:
+        if self._status_line_present and TTY:
             write_to_stream(STDOUT_WRITER, "\r\033[K")
+            self._status_line_present = False
 
     def _signal_handler_thread_body(self):
         while self._active:
             self.progress_show()
+            if not self._waiting_for_input:  # do not block and ignore SIGINT while .ask()ing
+                with self.lock:
+                    self._clear_last_job()
+                    self._write_current_job()
             if QUIT_EVENT.is_set():
                 if SHUTDOWN_EVENT_HARD.wait(0.1):
                     self.stderr(_("{x} {signal}  cleanup interrupted, exiting...").format(
@@ -342,6 +384,8 @@ class IOManager(object):
                         except ProcessLookupError:
                             pass
                     self._clear_last_job()
+                    if TTY:
+                        write_to_stream(STDOUT_WRITER, SHOW_CURSOR)
                     _exit(1)
             else:
                 if SHUTDOWN_EVENT_SOFT.wait(0.1):
@@ -354,6 +398,12 @@ class IOManager(object):
                         x=blue("i"),
                     ))
 
+    def _spinner_character(self):
+        if time() - self._last_spinner_update > 0.2:
+            self._last_spinner_update = time()
+            self._last_spinner_character = next(self._spinner)
+        return self._last_spinner_character
+
     def _write(self, msg, append_newline=True, err=False):
         if not self._active:
             return
@@ -366,6 +416,20 @@ class IOManager(object):
 
     def _write_current_job(self):
         if self.jobs and TTY:
-            write_to_stream(STDOUT_WRITER, inverse("{} ".format(self.jobs[-1])[:term_width() - 1]))
+            line = "{} ".format(blue(self._spinner_character()))
+            # must track line length manually as len() will count ANSI escape codes
+            visible_length = 2
+            try:
+                progress = (self.progress / float(self.progress_total))
+            except ZeroDivisionError:
+                pass
+            else:
+                progress_text = "{:.1f}%  ".format(progress * 100)
+                line += bold(progress_text)
+                visible_length += len(progress_text)
+            line += self.jobs[-1][:term_width() - 1 - visible_length]
+            write_to_stream(STDOUT_WRITER, line)
+            self._status_line_present = True
+
 
 io = IOManager()
