@@ -79,7 +79,7 @@ def test_encrypt_file_base64(tmpdir):
 def test_format_password(tmpdir):
     make_repo(tmpdir)
 
-    stdout, stderr, rcode = run("bw debug -c 'print(repo.vault.format(\"format: {}\", repo.vault.password_for(\"testing\")))'", path=str(tmpdir))
+    stdout, stderr, rcode = run("bw debug -c 'print(repo.vault.password_for(\"testing\").format_into(\"format: {}\"))'", path=str(tmpdir))
     assert stdout == b"format: faCTT76kagtDuZE5wnoiD1CxhGKmbgiX\n"
     assert stderr == b""
     assert rcode == 0
